from netgen.geom2d import unit_square
from ngsolve import *

mesh = Mesh (unit_square.GenerateMesh(maxh=0.05))

fes = L2(mesh, order=2)

b = CoefficientFunction( (y-0.5,0.5-x) )
n = specialcf.normal(mesh.dim)
bn = b*n

ubnd = CoefficientFunction(0)

u = fes.TrialFunction()
uhat = u.Other(bnd=ubnd)
v = fes.TestFunction()
vhat = u.Other()

# def f(u):
#     return u * b

# def fhatn(u):
#     return bn*IfPos(bn, u, u.Other(bnd=ubnd))

def max(a,b):
    return IfPos(a-b,a,b)

def g(u):
    return CoefficientFunction((max(u,0),max(u,0)))

def gn(u):
    return InnerProduct(g(u),n)

def avg(u):
    return 0.5*u+0.5*u.Other(bnd=ubnd)

def f(u):
    return u * g(u)

# def fhatn(u):
#     return IfPos(gn(avg(u)), gn(avg(u))*u, gn(avg(u))*u.Other(bnd=ubnd))



def fmax(a,b):
    return sqrt(2)*max(a*a,b*b)

def fhatn(u):
    return (0.5*f(u)+0.5*f(u.Other(bnd=ubnd)) + fmax(u,uhat)/2*(u-uhat)*n)*n


a = BilinearForm(fes)
a += SymbolicBFI (- f(u)*grad(v))
a += SymbolicBFI (fhatn(u) * v, element_boundary=True)

u = GridFunction(fes)
u.Set(exp (-40 * ( (x-0.2)*(x-0.2) + (y-0.2)*(y-0.2) )))

w = u.vec.CreateVector()

Draw (u, autoscale=False, sd=2)

t = 0
tau = 1e-3
tend = 10

with TaskManager():
    while t < tend:
        a.Apply (u.vec, w)
        fes.SolveM (rho=CoefficientFunction(1), vec=w)
        u.vec.data -= tau * w
        t += tau
        Redraw()
        # input("")
    
